Attribute VB_Name = "modListBox"

' Author:       M. Ruhwald
' Erstellt:     01.12.2007 (20:55)
' Gendert:     04.12.2007 (17:57)
' Beschreibung:

' Funktions-Modul fr LISTBOX-Controls
' ------------------------------------

' ListBox-Controls in VB6 knnen nur bis
' zu 32767 Eintrge verarbeiten (Integer).
' Versucht man Index-Werte (ListIndex) oberhalb
' dieser Grenze zu verwenden, gibt es Laufzeitfehler.
'
' Dieses Manko kann umgangen werden, wenn dieses Modul
' verwendet wird. Es erlaubt eine Index-Grende vom Typ Long.
' Somit ist es mglich, die Anzahl der Listen-Eintrge
' auf eine Million und mehr zu erhhen.


Option Explicit

' ***********************************************
' *             API-Deklaration                 *
' ***********************************************

' Messages fr ListBox
Private Const LB_FINDSTRING As Long = &H18F
Private Const LB_FINDSTRINGEXACT As Long = &H1A2
Private Const LB_SELECTSTRING As Long = &H18C
Private Const LB_GETCURSEL As Long = &H188
Private Const LB_SETCURSEL As Long = &H186
Private Const LB_GETTEXTLEN As Long = &H18A
Private Const LB_GETTEXT As Long = &H189
Private Const LB_GETCOUNT As Long = &H18B
Private Const LB_DELETESTRING As Long = &H182
Private Const LB_ADDFILE As Long = &H196
Private Const LB_ADDSTRING As Long = &H180
Private Const LB_CTLCODE As Long = 0&
Private Const LB_DIR As Long = &H18D
Private Const LB_GETHORIZONTALEXTENT As Long = &H193
Private Const LB_GETITEMDATA As Long = &H199
Private Const LB_GETITEMHEIGHT As Long = &H1A1
Private Const LB_GETITEMRECT As Long = &H198
Private Const LB_GETLOCALE As Long = &H1A6
Private Const LB_GETSEL As Long = &H187
Private Const LB_GETSELCOUNT As Long = &H190
Private Const LB_ITEMFROMPOINT As Long = &H1A9
'Private Const LB_Okay As Long = 0
Private Const LB_SELITEMRANGE As Long = &H19B
Private Const LB_SELITEMRANGEEX As Long = &H183
Private Const LB_SETTABSTOPS As Long = &H192
Private Const LB_SETTOPINDEX As Long = &H197
Private Const LB_SETITEMHEIGHT As Long = &H1A0
Private Const LB_INSERTSTRING As Long = &H181
Private Const LB_SETITEMDATA As Long = &H19A

Private Type POINTAPI
    X As Long
    Y As Long
End Type


Private Declare Function SendMessage Lib "user32.dll" Alias "SendMessageA" ( _
     ByVal hwnd As Long, _
     ByVal wMsg As Long, _
     ByVal wParam As Long, _
     ByRef lParam As Any) As Long



' ***********************************************
' *     Benutzer-Funktionen ListBox-Control     *
' ***********************************************


' Sucht String in ListBox, gibt Index zurck
' wenn bereinstimmung gefunden, sonst -1
' Der Eintrag wird Markiert
Public Function LBSelectListItem(Obj As ListBox, StartIndex As Long, FindString As String) As Long
    LBSelectListItem = SendMessage(Obj.hwnd, LB_SELECTSTRING, 0, ByVal FindString)
End Function

' Sucht exakten String in ListBox, gibt Index bei
' bereinstimmung zurck, sonst -1
' Der Eintrag wird nicht markiert
Public Function LBFindExactItem(Obj As ListBox, StartIndex As Long, FindString As String) As Long
    LBFindExactItem = SendMessage(Obj.hwnd, LB_FINDSTRINGEXACT, StartIndex, ByVal FindString)
End Function

' Sucht (nicht exakten) String in ListBox, gibt Index bei
' bereinstimmung zurck, sonst -1
' Der Eintrag wird nicht Markiert
Public Function LBFindItem(Obj As ListBox, StartIndex As Long, FindString As String) As Long
    LBFindItem = SendMessage(Obj.hwnd, LB_FINDSTRING, StartIndex, ByVal FindString)
End Function

' ListIndex von ListBox ermitteln
' Gibt den aktuell selektierten Index zurck
Public Function LBGetListIndex(Obj As ListBox) As Long
    LBGetListIndex = SendMessage(Obj.hwnd, LB_GETCURSEL, 0, ByVal 0&)
End Function

' ListIndex von ListBox setzen
' setzt den ListIndex und markiert den Eintrag
' Ein Click wird nicht ausgelst!
Public Sub LBSetListIndex(Obj As ListBox, NewIndex As Long)
    Call SendMessage(Obj.hwnd, LB_SETCURSEL, NewIndex, ByVal 0&)
End Sub

' Anzahl List-Eintrge ermitteln (ListCount)
Public Function LBGetListCount(Obj As ListBox) As Long
    LBGetListCount = SendMessage(Obj.hwnd, LB_GETCOUNT, 0, ByVal 0&)
End Function

' Liest den Text in einer ListBox aus mit dem
' angegebenen Index aus und gibt diesen zurck
Public Function LBGetItemText(Obj As ListBox, Index As Long) As String
    Dim Buffer As String
    Dim TextLen As Long
    TextLen = SendMessage(Obj.hwnd, LB_GETTEXTLEN, Index, ByVal 0&)
    If TextLen > 0 Then
        Buffer = Space(TextLen)
        Call SendMessage(Obj.hwnd, LB_GETTEXT, Index, ByVal Buffer)
    End If
    LBGetItemText = Buffer
End Function

' Lscht den Text aus der ListBox
' Gibt bei Erfolg die Anzahl der verbleibenden
' Eintrge in der Liste zurck, bei Fehler -1
Public Function LBDeleteItem(Obj As ListBox, Index As Long) As Long
    LBDeleteItem = SendMessage(Obj.hwnd, LB_DELETESTRING, Index, ByVal 0&)
End Function
 
' Fgt ein Eintrag an die Position 'Index' ein
' Der eingefgte Text wird nicht matkiert.
Public Sub LBInsertItem(Obj As ListBox, Index As Long, Text As String)
    Call SendMessage(Obj.hwnd, LB_INSERTSTRING, Index, ByVal Text)
End Sub

' Hngt ans Ende der Listbox einen Eintrag an.
' Liefert die Position zurck, an der der text eingefgt wurde
' Der hinzugefgte Text wird nicht markiert
Public Function LBAppendItem(Obj As ListBox, Text As String) As Long
    LBAppendItem = SendMessage(Obj.hwnd, LB_ADDSTRING, 0, ByVal Text)
End Function

' Ersetzt den Item-Text an der Position 'Index' mit 'NewText'
' Der Eintrag wird markiert!
Public Sub LBSetItemText(Obj As ListBox, Index As Long, NewText As String)
    ' zunchst Eintrag lschen
    If SendMessage(Obj.hwnd, LB_DELETESTRING, Index, ByVal 0&) > 0 Then
        ' und neuen Eintrag an diese Position wieder einfgen
        Call SendMessage(Obj.hwnd, LB_INSERTSTRING, Index, ByVal NewText)
        ' und noch selectieren
        Call SendMessage(Obj.hwnd, LB_SETCURSEL, Index, ByVal 0)
    End If
End Sub

' Ermittelt den benutzerdefinierten Wert ItemData
' Der Wert wird mit LBSetItemData() zugewiesen
Public Function LBGetItemData(Obj As ListBox, Index As Long) As Long
    LBGetItemData = SendMessage(Obj.hwnd, LB_GETITEMDATA, Index, ByVal 0&)
End Function

' setzt den benutzerdefinierten Wert ItemData
Public Sub LBSetItemData(Obj As ListBox, Index As Long, ItemData As Long)
    Call SendMessage(Obj.hwnd, LB_SETITEMDATA, Index, ByVal ItemData)
End Sub
